<?php
if (!defined('ABSPATH')) {
    exit;
}

add_action('plugins_loaded', 'register_fpayment_gateway');

function register_fpayment_gateway()
{
    if (!class_exists('WC_Payment_Gateway')) {
        return;
    }

    class WC_Fpayment_Gateway extends WC_Payment_Gateway
    {
        public function __construct()
        {
            $this->id = 'fpayment';
            $this->icon = '';
            $this->has_fields = false; // Không hiển thị fields thêm
            $this->method_title = __('FPayment', 'fpayment');
            $this->method_description = __('Pay with USDT', 'fpayment');

            $this->order_button_text = __( 'Pay now', 'fpayment' );

            $this->init_form_fields();
            $this->init_settings();

            $this->title = $this->get_option('title');
            $this->description = $this->get_option('description');
            $this->enabled = $this->get_option('enabled');

            add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
            add_action('woocommerce_thankyou_' . $this->id, array($this, 'mess_thank_you'));
        }

        public function init_form_fields()
        {
            $this->form_fields = array(
                'enabled' => array(
                    'title'   => __('Enable/Disable', 'fpayment'),
                    'type'    => 'checkbox',
                    'label'   => __('Enable Fpayment Payment', 'fpayment'),
                    'default' => 'yes',
                ),
                'title' => array(
                    'title'       => __('Title', 'fpayment'),
                    'type'        => 'text',
                    'description' => __('Display name for customers', 'fpayment'),
                    'default'     => __('FPayment - Pay with USDT', 'fpayment'),
                    'desc_tip'    => true,
                ),
                'description' => array(
                    'title'       => __('Description', 'fpayment'),
                    'type'        => 'textarea',
                    'description' => __('Description under payment method.', 'fpayment'),
                    'default'     => __('You will be redirected to fpayment.net to complete the payment. High security - Use with confidence.', 'fpayment'),
                ),
                'merchant_id' => array(
                    'title'       => __('Store ID', 'fpayment'),
                    'type'        => 'password',
                    'description' => __('Store ID', 'fpayment'),
                    'default'     => '',
                    'desc_tip'    => true,
                ),
                'api_key' => array(
                    'title'       => __('Store API KEY', 'fpayment'),
                    'type'        => 'password',
                    'description' => __('Store API KEY', 'fpayment'),
                    'default'     => '',
                    'desc_tip'    => true,
                ),
                'rate_usdt' => array(
                    'title'       => __('Exchange rate for 1 USDT', 'fpayment'),
                    'type'        => 'number',
                    'description' => __('Exchange rate for 1 USDT to VND', 'fpayment'),
                    'default'     => 25800,
                    'desc_tip'    => false,
                ),
                'min_pay' => array(
                    'title'       => __('Minimum payment', 'fpayment'),
                    'type'        => 'number',
                    'description' => __('Enter the minimum payment amount for this payment gateway (VND)', 'fpayment'),
                    'default'     => 25000,
                    'desc_tip'    => false,
                ),
                'status_order' => array(
                    'title' =>  __('Order status', 'fpayment'),
                    'type' => 'select',
                    'options' => wc_get_order_statuses(),
                    'description' => __('Order status after payment', 'fpayment'),
                    'default' => 'wc-received-payment',
                    'desc_tip'    => false,
                ),
            );
        }

        function callback_url(){
            return untrailingslashit(get_rest_url()) . '/fpayment/v1/callback-url';
        }

        function success_url($order){
            return add_query_arg(array(
                'status' => 'success',
            ), $order->get_checkout_order_received_url());
        }

        function cancel_url($order){
            return add_query_arg(array(
                'status' => 'error',
            ), $order->get_checkout_order_received_url());
        }

        public function process_payment($order_id)
        {

            $order = wc_get_order($order_id);

            $order->add_order_note(__('Pay via FPayment.', 'fpayment'));
            $order->update_status('pending', __('Awaiting payment.', 'fpayment'));

            wc_reduce_stock_levels($order_id);

            $rate_usdt = $this->get_option('rate_usdt');

            $amount = ($rate_usdt) ? round($order->get_total() / $rate_usdt, 3) : $order->get_total();
            $request_id = $order->get_id() .'-' . wp_rand(100,999) . wp_generate_password(12, false);

            $api_url = 'https://app.fpayment.net/api/AddInvoice';
            $payload = array(
                'merchant_id'     => $this->get_option('merchant_id'),
                'api_key'         => $this->get_option('api_key'),
                'name'            => sprintf(__('Invoice #%s', 'fpayment'), $order->get_id()),
                'description'     => sprintf(__('Pay invoice #%s', 'fpayment'), $order->get_id()),
                'amount'          => $amount,
                'request_id'      => $request_id,
                'callback_url'    => $this->callback_url(),
                'success_url'     => $this->success_url($order),
                'cancel_url'      => $this->cancel_url($order)
            );

            $response = wp_remote_post( $api_url, array(
                'body'      => $payload,
                //'timeout'   => 30,
                'sslverify' => true,
                'headers'   => array(),
            ));

            $order->update_meta_data('fpayment_payload', $payload);

            /*ob_start();
            print_r($response);
            error_log(ob_get_clean());*/

            if ( is_wp_error( $response ) ) {
                $error_message = $response->get_error_message();
                $note = sprintf(__('Unable to pay via USDT due to: %s', 'fpayment') , $error_message);
                $order->add_order_note( $note );
                wc_add_notice( $note, 'error' );
                return array(
                    'result' 	=> 'success',
                    'redirect'	=> $order->get_checkout_payment_url()
                );
            }

            $respon_code = wp_remote_retrieve_response_code($response);

            if($respon_code != 200){
                $note = __('Error connecting to fpayment.net', 'fpayment');
                $order->add_order_note( $note );
                wc_add_notice( $note, 'error' );
                return array(
                    'result' 	=> 'success',
                    'redirect'	=> $order->get_checkout_payment_url()
                );
            }

            $result = json_decode( wp_remote_retrieve_body( $response ), true );
            $message = isset( $result['msg'] ) ? sanitize_text_field(wp_unslash($result['msg'])) : '';

            $order->update_meta_data('fpayment_request_id', $request_id);
            $order->update_meta_data('fpayment_result', $result);

            if ( isset( $result['status'] ) && $result['status'] === 'success' ) {

                $trans_id       = isset($result['data']['trans_id']) ? sanitize_text_field($result['data']['trans_id']) : '';
                $amount         = isset($result['data']['amount']) ? floatval($result['data']['amount']) : 0;
                $status         = isset($result['data']['status']) ? sanitize_text_field($result['data']['status']) : '';
                $url_payment    = isset($result['data']['url_payment']) ? esc_url($result['data']['url_payment']) : '';

                return array(
                    'result'   => 'success',
                    'redirect' => $url_payment ? $url_payment : $this->get_return_url($order),
                );
            }

            $note = sprintf(__('Unable to pay via USDT due to: %s', 'fpayment') , $message);
            $order->add_order_note( $note );
            wc_add_notice( $note, 'error' );
            
            return array(
                'result'   => 'success',
                'redirect' => $order->get_checkout_payment_url(),
            );
        }

        public function is_available() {
            $is_available = ( 'yes' === $this->enabled );

            if ( WC()->cart && 0 < $this->get_order_total() && 0 < $this->max_amount && $this->max_amount < $this->get_order_total() ) {
                $is_available = false;
            }

            if(WC()->cart && $this->get_order_total() < $this->get_option('min_pay')  || !$this->get_option('rate_usdt') || !$this->get_option('merchant_id') || !$this->get_option('api_key')) {
                $is_available = false;
            }

            return $is_available;

        }

        function mess_thank_you($order_id){
            $status = isset($_GET['status']) ? sanitize_text_field(wp_unslash($_GET['status'])) : '';
            $order = wc_get_order($order_id);

            if(!$order || is_wp_error($order) || !$status) return '';

            $order_status = $order->get_status();
            $default_status_order = $this->get_option('status_order');

            $title = __('Payment Successful!', 'fpayment');
            $sub_title = '';
            $class_box = 'fpayment_success';
            $icon = '<svg xmlns="http://www.w3.org/2000/svg" clip-rule="evenodd" fill-rule="evenodd" image-rendering="optimizeQuality" shape-rendering="geometricPrecision" text-rendering="geometricPrecision" viewBox="0 0 21.0001 21.0001"><g id="图层_x0020_1"><path d="m0 0h21v21h-21z" fill="none"/><path d="m10.5038 1.31775c5.07328 0 9.1876 4.11432 9.1876 9.1876s-4.11432 9.1876-9.1876 9.1876-9.1876-4.11432-9.1876-9.1876 4.11432-9.1876 9.1876-9.1876zm-1.90792 12.1718-2.24935-2.25121c-.38321-.38344-.38329-1.00872 0-1.39208.38337-.38329 1.01143-.38089 1.39201 0l1.58578 1.58702 3.94488-3.94488c.38337-.38337 1.00872-.38337 1.39201 0 .38337.38329.38282 1.00918 0 1.39201l-4.64201 4.64201c-.38282.38282-1.00872.38337-1.39201 0-.01077-.01077-.02116-.0217-.03131-.03286z" fill="#fff"/></g></svg>';

            if($status == 'error'){
                $title = __('Payment cancelled!', 'fpayment');
                $sub_title = __('User has cancelled payment!', 'fpayment');
                $class_box = 'fpayment_error';
                $icon = '<svg height="512" viewBox="0 0 512 512" width="512" xmlns="http://www.w3.org/2000/svg"><g id="Layer_2" data-name="Layer 2"><g id="close"><circle id="background" cx="256" cy="256" fill="#f44336" r="256"/><path d="m348.6 391a42.13 42.13 0 0 1 -30-12.42l-62.6-62.58-62.6 62.61a42.41 42.41 0 1 1 -60-60l62.6-62.61-62.61-62.6a42.41 42.41 0 0 1 60-60l62.61 62.6 62.6-62.61a42.41 42.41 0 1 1 60 60l-62.6 62.61 62.61 62.6a42.41 42.41 0 0 1 -30 72.4z" fill="#fff"/></g></g></svg>';
            }elseif($order_status != 'completed' && $order_status != str_replace('wc-', '', $default_status_order)){
                $title = __('Payment received!', 'fpayment');
                $sub_title = __('Please wait a moment. We are processing your order!', 'fpayment');
                $class_box = 'fpayment_pending';
                $icon = '<svg height="512" viewBox="0 0 64 64" width="512" xmlns="http://www.w3.org/2000/svg"><g id="Layer_35" data-name="Layer 35"><path d="m32 3.4a28.59 28.59 0 0 0 -28.6 28.6 28.59 28.59 0 0 0 28.6 28.6 28.59 28.59 0 0 0 28.6-28.6 28.59 28.59 0 0 0 -28.6-28.6zm0 49.2a4.32 4.32 0 1 1 4.31-4.31 4.32 4.32 0 0 1 -4.31 4.31zm5.23-35.6-1.63 22a.6.6 0 0 1 -.6.56h-6a.6.6 0 0 1 -.6-.56l-1.63-22a5.24 5.24 0 1 1 10.46 0z" fill="#ffc048"/></g></svg>';
            }
            ?>
            <div class="fpayment_status_mess <?php echo $class_box;?>">
                <div class="fpayment_icon"><?php echo $icon;?></div>
                <strong><?php echo $title;?></strong>
                <?php if($sub_title):?><span><?php echo $sub_title;?></span><?php endif;?>
            </div>
            <style>
                .fpayment_icon svg {
                    width: 60px;
                    height: 60px;
                    animation: pulse 1.5s infinite;
                }

                @keyframes pulse {
                    0% {
                        transform: scale(1);
                    }
                    50% {
                        transform: scale(1.1);
                    }
                    100% {
                        transform: scale(1);
                    }
                }

                .fpayment_status_mess {
                    box-shadow: 0 0 #0000, 0 0 #0000, 0 20px 25px -5px rgb(0 0 0 / 0.1), 0 8px 10px -6px rgb(0 0 0 / 0.1);
                    background-image: linear-gradient(to right, rgb(74, 222, 128), rgb(22, 163, 74));
                    text-align: center;
                    padding: 32px;
                    border-radius: 10px;
                    margin: 0 0 30px;
                    color: #fff;
                }

                .fpayment_status_mess > strong {
                    font-size: 1.5em;
                    display: block;
                }
                .fpayment_status_mess.fpayment_error {
                    background-image: linear-gradient(to right, rgb(240 139 132), rgb(237 89 78));
                }
                .fpayment_status_mess.fpayment_pending {
                    background-image: linear-gradient(to right, rgb(69 57 34), rgb(103 82 42));
                }
            </style>
            <?php
        }
    }
}

add_filter('woocommerce_payment_gateways', 'add_custom_payment_gateway');

function add_custom_payment_gateway($gateways)
{
    $gateways[] = 'WC_Fpayment_Gateway';
    return $gateways;
}
