<?php
/**
 * Plugin Name:       FPAYMENT.NET - Integrate USDT Payment
 * Plugin URI:        https://app.fpayment.net
 * Description:       Integrate USDT payments via fpayment.net into your website.
 * Version:           1.0.0
 * Author:            fpayment.net
 * Author URI:        https://app.fpayment.net/
 * Text Domain:       fpayment
 * Domain Path:       /languages
 */

defined( 'ABSPATH' ) or die( 'No script kiddies please!' );

class Fpayment_Class
{

    protected static $_instance = null;

    public static function instance() {
        if ( is_null( self::$_instance ) ) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }

    public function __construct() {

        $this->includes();

        add_action('init', array($this, 'register_received_payment_order_status'));
        add_filter('wc_order_statuses', array($this, 'add_received_payment_to_order_statuses'));
        add_action('woocommerce_order_status_changed', array($this, 'send_custom_email_notifications'), 10, 4 );

        add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), array( $this, 'add_action_links' ), 10, 2 );

        add_action( 'woocommerce_blocks_loaded', array($this, 'register_payment_method_type') );

        add_action( 'plugins_loaded', array($this, 'load_textdomain') );

    }

    function includes(){
        include_once plugin_dir_path(__FILE__) . 'includes/rest-api.php';
        include_once plugin_dir_path(__FILE__) . 'includes/class-fpayment-method.php';
    }

    function register_received_payment_order_status()
    {
        register_post_status('wc-received-payment', array(
            'label' => __('Paid', 'fpayment'),
            'public' => true,
            'exclude_from_search' => false,
            'show_in_admin_all_list' => true,
            'show_in_admin_status_list' => true,
            'label_count' => _n_noop('Paid <span class="count">(%s)</span>', 'Paid <span class="count">(%s)</span>', 'fpayment')
        ));
    }

    function add_received_payment_to_order_statuses($order_statuses)
    {
        if(!isset($order_statuses['wc-received-payment'])) {
            $new_order_statuses = array();
            foreach ($order_statuses as $key => $status) {
                $new_order_statuses[$key] = $status;
                if ('wc-on-hold' === $key) {
                    $new_order_statuses['wc-received-payment'] = __('Paid', 'fpayment');
                }
            }
            return $new_order_statuses;
        }
        return $order_statuses;
    }

    function send_custom_email_notifications($order_id, $old_status, $new_status, $order){
        if ( $new_status == 'received-payment' ){
            $wc_emails = WC()->mailer()->get_emails();
            $customer_email = $order->get_billing_email();
        }
        if ( $new_status == 'received-payment' ) {
            //$wc_emails['WC_Email_Customer_Completed_Order']->recipient = $customer_email;
            $wc_emails['WC_Email_Customer_Completed_Order']->trigger( $order_id );
        }
    }

    public function add_action_links( $links, $file ) {
        if ( strpos( $file, 'fpayment.php' ) !== false ) {
            $settings_link = '<a href="' . admin_url( 'admin.php?page=wc-settings&tab=checkout&section=fpayment' ) . '" title="'.__('Settings').'">' . __( 'Settings' ) . '</a>';
            array_unshift( $links, $settings_link );
        }
        return $links;
    }

    function register_payment_method_type(){
        if ( ! class_exists( 'Automattic\WooCommerce\Blocks\Payments\Integrations\AbstractPaymentMethodType' ) ) {
            return;
        }
        require_once plugin_dir_path(__FILE__) . 'includes/class-fpayment-block-checkout.php';
        add_action(
            'woocommerce_blocks_payment_method_type_registration',
            function( Automattic\WooCommerce\Blocks\Payments\PaymentMethodRegistry $payment_method_registry ) {
                $payment_method_registry->register( new WC_Fpayment_Blocks );
            }
        );
    }

    function load_textdomain(){
        $locale = determine_locale();
        $locale = apply_filters( 'plugin_locale', $locale, 'fpayment' );

        unload_textdomain( 'fpayment' );
        $loaded = load_textdomain( 'fpayment', WP_LANG_DIR . '/plugins/fpayment-' . $locale . '.mo' );
        if(!$loaded) {
            load_textdomain('fpayment', plugin_dir_path(__FILE__) . 'languages/fpayment-' . $locale . '.mo');
        }
    }

}

function fpayment() {
    return Fpayment_Class::instance();
}
fpayment();

register_activation_hook(__FILE__, 'fpayment_plugin_activate');
function fpayment_plugin_activate() {
    flush_rewrite_rules();
}