<?php
if (!defined('ABSPATH')) {
    exit;
}

add_action( 'rest_api_init', 'fpayment_ipn_rest_routes' );
function fpayment_ipn_rest_routes(){
    $change_date = new FPayment_IPN_Route();
    $change_date->register_routes();
}

class FPayment_IPN_Route extends WP_REST_Controller {
    public function __construct() {
        $this->namespace         = 'fpayment/v1';
        $this->rest_base         = 'callback-url';
    }
    public function register_routes() {
        register_rest_route(
            $this->namespace,
            '/'.$this->rest_base,
            array(
                array(
                    'methods'             => array( WP_REST_Server::READABLE, WP_REST_Server::CREATABLE ),
                    'callback'            => array( $this, 'callback_url' ),
                    'permission_callback' => '__return_true',
                    //'args'                => $this->get_collection_params(),
                ),
                'schema' => array( $this, 'get_public_item_schema' ),
            )
        );

        register_rest_route(
            $this->namespace,
            '/success-url',
            array(
                array(
                    'methods'             => array( WP_REST_Server::READABLE, WP_REST_Server::CREATABLE ),
                    'callback'            => array( $this, 'success_url' ),
                    'permission_callback' => '__return_true',
                    //'args'                => $this->get_collection_params(),
                ),
                'schema' => array( $this, 'get_public_item_schema' ),
            )
        );

        register_rest_route(
            $this->namespace,
            '/cancel-url',
            array(
                array(
                    'methods'             => WP_REST_Server::READABLE,
                    'callback'            => array( $this, 'cancel_url' ),
                    'permission_callback' => '__return_true',
                    //'args'                => $this->get_collection_params(),
                ),
                'schema' => array( $this, 'get_public_item_schema' ),
            )
        );
    }
    public function get_items_permissions_check( $request ) {
        return true;
    }

    public function get_item_permissions_check( $request ) {
        return $this->get_items_permissions_check( $request );
    }

    public function callback_url( $request ) {

        $request_data = $_GET;

        if (!isset($request_data['request_id'], $request_data['merchant_id'], $request_data['api_key'], $request_data['received'], $request_data['status'])) {
            http_response_code(400);
            echo json_encode([
                'status' => 'error',
                'message' => 'Thiếu tham số callback.'
            ]);
            exit;
        }

        $request_id = sanitize_text_field(wp_unslash($request_data['request_id']));
        $merchant_id = sanitize_text_field(wp_unslash($request_data['merchant_id']));
        $api_key = sanitize_text_field(wp_unslash($request_data['api_key']));
        $amount = floatval($request_data['amount']);
        $received = floatval($request_data['received']);
        $status = sanitize_text_field(wp_unslash($request_data['status']));
        $from_address = isset($request_data['from_address']) ? sanitize_text_field(wp_unslash($request_data['from_address'])) : null;
        $transaction_id = isset($request_data['transaction_id']) ? sanitize_text_field(wp_unslash($request_data['transaction_id'])) : null;

        $payment_setting = get_option('woocommerce_fpayment_settings');
        $expected_merchant_id = isset($payment_setting['merchant_id']) ? sanitize_text_field($payment_setting['merchant_id']) : '';
        $expected_api_key = isset($payment_setting['api_key']) ? sanitize_text_field($payment_setting['api_key']) : '';
        $status_order = isset($payment_setting['status_order']) ? sanitize_text_field($payment_setting['status_order']) : '';

        if ($merchant_id !== $expected_merchant_id || $api_key !== $expected_api_key) {
            http_response_code(403);
            echo json_encode([
                'status' => 'error',
                'message' => 'Merchant ID hoặc API Key không hợp lệ.'
            ]);
            exit;
        }

        if($amount > $received){
            http_response_code(403);
            echo json_encode([
                'status' => 'error',
                'message' => 'Số tiền đã nhận không hợp lệ.'
            ]);
            exit;
        }

        $order_id = (array) explode('-', $request_id);
        $order_id = array_shift($order_id);

        $order = wc_get_order($order_id);

        if(!$order || is_wp_error($order)){
            http_response_code(403);
            echo json_encode([
                'status' => 'error',
                'message' => 'Đơn hàng không hợp lệ.'
            ]);
            exit;
        }

        switch ($status) {
            case 'waiting':
                // Giao dịch đang chờ thực hiện
                // Thêm code để xử lý trạng thái đang chờ
                break;
            case 'expired':
                $order->update_status('wc-cancelled');
                break;
            case 'completed':
                $order->update_status($status_order);
                break;
            default:
                http_response_code(400);
                echo json_encode([
                    'status' => 'error',
                    'message' => 'Trạng thái giao dịch không hợp lệ.'
                ]);
                exit;
        }

        $order->save();

        http_response_code(200);
        echo json_encode([
            'status' => 'success',
            'message' => 'Callback đã được xử lý thành công.'
        ]);

        die();
    }

    public function success_url( $request ) {

        die();

    }

}